#include "stdafx.h"
#include "TSDescrambler.h"


void CTSDescrambler::SetTunerInterface(IDY_UD200* dy_ud200)
{
	this->pTuner = dy_ud200;
}

void CTSDescrambler::SetPacketFilterInterface(CTSPacketFilter* filter)
{
	this->pFilter = filter;
}

void CTSDescrambler::InitializeSystemKey(const BYTE km[32], const BYTE cbc[8])
{
	this->decoder.Initialize(km, cbc);
}

/*
ECMZNVe[u̍\
   0    1    2    3    4    5    6    7    8  ...  x-4  x-3  x-2  x-1    x   
0x82 0x00 0x27 0x00 0x00 0x00 0x00 0x00 0x00  ... 0x00 0x00 0x00 0x00 0x00
                |\\\\\\\ZNVTCY͈̔́\\\\\\\\|
|\\\\\\\\\CRCvZ͈-\\\\\\\\\\|\\CRC\\\|
|\\\\ZNVwb_\\\\\|\ECM{́\|\\CRC\\\|
*/
bool CTSDescrambler::ParsePacket(CTSPacket& packet)
{
	static const BYTE EcmReceiveCmd[] = {0x90U, 0x34U, 0x00U, 0x00U};

	if(!this->pTuner)
		return false;

	if(packet.IsEmpty())
		return false;
	if(packet.LockByte() != 0x47)
		return false;
	if(packet.ErrorBit())
		return false;

	//ECMpP̏
	if(this->pFilter->IsECM(packet))
	{
		this->accECM.ParsePacket(packet);

		if(this->accECM.IsAccumulated())
		{
			//ECMXV
			BYTE cmd[256], res[128];
			DWORD dw;
			BYTE *pEcmData;
			unsigned int dwEcmSize;
			
			this->accECM.GetSection(&pEcmData, &dwEcmSize);

			//ECMȂقǂł DY-UD200̐
			if(dwEcmSize-9 > 109)
				return false;

			//e[uʎq̃`FbN
			if(pEcmData[0] != 0x82)
				return false;

			if(CalcTsCrc32(pEcmData, 3+dwEcmSize))
			{
				//CRCsv
				return false;
			}

			dwEcmSize=pEcmData[2]-9;
			pEcmData+=8;

			if(memcmp(this->oldECM, pEcmData, dwEcmSize) == 0)
				return true;//ECM̍XV

			//R}h\z
			memcpy(cmd,  EcmReceiveCmd, sizeof(EcmReceiveCmd));			// CLA, INS, P1, P2
			cmd[sizeof(EcmReceiveCmd)] = (BYTE)dwEcmSize;				// COMMAND DATA LENGTH
			memcpy(cmd+sizeof(EcmReceiveCmd)+1, pEcmData, dwEcmSize);	// ECM
			cmd[sizeof(EcmReceiveCmd) + dwEcmSize + 1] = 0x00;			// RESPONSE DATA LENGTH

			if(!this->pTuner->SendToBCASAndGetResponse(cmd, sizeof(EcmReceiveCmd) + dwEcmSize + 2, res, 128, &dw))
				return false;

			if(dw != 25)
				return false;

			switch(((WORD)res[4] << 8) | (WORD)res[5]){
				case 0x0200:	// Payment-deferred PPV
				case 0x0400:	// Prepaid PPV
				case 0x0800:	// Tier
					memcpy(this->oldECM, pEcmData, dwEcmSize);
					this->decoder.SetScrambleKey(res+6);
					return true;
				// LȊO(s)
				default:
					return false;
			}
		}
	}

	//B25fR[h
	BYTE *dd, size;
	packet.GetPayload(&dd, &size);
	if(!(this->decoder.*DecodeFuncPtr)(dd, size, packet.PayloadScrambleControl()))
		return false;

	*(packet.GetPacket()+3) &= 0x3f;
	return true;
}

void CTSDescrambler::SetSSE2(bool f)
{
	if(f && this->decoder.IsSSE2Available())
		this->DecodeFuncPtr = &CMulti2Decoder::DecodeSSE2;
	else
		this->DecodeFuncPtr = &CMulti2Decoder::Decode;
}

CTSDescrambler::CTSDescrambler():pTuner(NULL)
{
	this->DecodeFuncPtr = &CMulti2Decoder::Decode;
}

