#include "stdafx.h"
#include "uusbb_libusb.h"


//libusb---------------------------------------------------------------------------------
bool CUUSBB_libusb::Open(const GUID &guid, unsigned short vendorId, unsigned short productId)
{
	struct usb_bus *busses, *bus;
	struct usb_device *dev;

	//libusb̏ƃoXXg̎擾
	usb_init();
	usb_find_busses();
	usb_find_devices();
	busses = usb_get_busses();

	//w肵VIDPIDŃoXXgfoCX
	for(bus=busses; bus; bus=bus->next){
		for(dev=bus->devices; dev; dev=dev->next) {
			if( (dev->descriptor.idVendor==vendorId) && (dev->descriptor.idProduct==productId) ){
				break;
			}
		}
		if(dev)
			break;
	}

	if(!dev)
	{
		//w肳ꂽVID,PIDȂ
		MessageBox(NULL, _T("USBfoCX܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return false;
	}

	//foCX̃I[vƊe평
	if( (udev=usb_open(dev))==NULL ){
		MessageBox(NULL, _T("usb_open()Ɏs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return false;
	}

	if( usb_set_configuration(udev,dev->config->bConfigurationValue)<0 ){
		MessageBox(NULL, _T("usb_set_configuration()Ɏs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return false;
	}

	if( usb_claim_interface(udev,dev->config->interface->altsetting->bInterfaceNumber)<0 ){
		MessageBox(NULL, _T("usb_claim_interface()Ɏs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return false;
	}
	opened = true;
	return true;
}

void CUUSBB_libusb::Close(void)
{
	if(usb_release_interface(udev, 0)){
		MessageBox(NULL, _T("usb_release_interface()Ɏs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
	}
	if( usb_close(udev)<0 ){
		MessageBox(NULL, _T("usb_close()Ɏs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
	}
}

bool CUUSBB_libusb::IsOpened(void)
{
	if(!udev)
		return false;

	return opened;
}

bool CUUSBB_libusb::ReadPipe(BYTE pipeId, BYTE* buffer, DWORD bufferSize, DWORD *transferredSize)
{
	int tdSize, i;

	if(!this->IsOpened())
		return false;

	i=0;
	while((tdSize = usb_bulk_read(udev, pipeId, (char*)(buffer), bufferSize, LOOPBACK_TIMEOUT))<0)
	{
		if(i++ == 10)
		{
			return false;
		}
		Sleep(1);
	}
	*transferredSize = tdSize;

	return true;
}

bool CUUSBB_libusb::WritePipe(BYTE pipeId, BYTE* buffer, DWORD bufferSize, DWORD *transferredSize)
{
	if(!this->IsOpened())
		return false;

	if((*transferredSize = usb_bulk_write(udev, pipeId, (char*)(buffer), bufferSize, LOOPBACK_TIMEOUT))<0)
		return false;
	return true;
}

CUUSBB_libusb::CUUSBB_libusb():opened(false),udev(NULL)
{
	this->dllHandle = LoadLibrary(_T("libusb0.dll"));
	if(this->dllHandle == NULL)
	{
		MessageBox(NULL, _T("libusb0.dll̃[hɎs܂B"), _T(__FILE__), MB_OK | MB_ICONERROR);
		return;
	}

	usb_init = (PUSB_INIT)GetProcAddress(this->dllHandle, "usb_init");
	usb_find_busses = (PUSB_FIND_BUSSES)GetProcAddress(this->dllHandle, "usb_find_busses");
	usb_find_devices = (PUSB_FIND_DEVICES)GetProcAddress(this->dllHandle, "usb_find_devices");
	usb_get_busses = (PUSB_GET_BUSSES)GetProcAddress(this->dllHandle, "usb_get_busses");
	usb_open = (PUSB_OPEN)GetProcAddress(this->dllHandle, "usb_open");
	usb_close = (PUSB_CLOSE)GetProcAddress(this->dllHandle, "usb_close");
	usb_set_configuration = (PUSB_SET_CONFIGURATION)GetProcAddress(this->dllHandle, "usb_set_configuration");
	usb_claim_interface = (PUSB_CLAIM_INTERFACE)GetProcAddress(this->dllHandle, "usb_claim_interface");
	usb_release_interface = (PUSB_RELEASE_INTERFACE)GetProcAddress(this->dllHandle, "usb_release_interface");
	usb_bulk_read = (PUSB_BULK_READ)GetProcAddress(this->dllHandle, "usb_bulk_read");
	usb_bulk_write = (PUSB_BULK_WRITE)GetProcAddress(this->dllHandle, "usb_bulk_write");

	opened = true;
}

CUUSBB_libusb::~CUUSBB_libusb()
{
	if(udev)
		usb_close(udev);

	FreeLibrary(this->dllHandle);
}
